@@ -17,7 +17,7 @@ use Carp;
 use Business::CompanyDesignator::Record;
 use Business::CompanyDesignator::SplitResult;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 has 'datafile' => ( is => 'ro', default => sub {
   # Development/test version
@@ -137,12 +137,22 @@ sub _build_regex {
   # RA constructor - case insensitive, with match tracking
   my $assembler = Regexp::Assemble->new->flags('i')->track(1);
 
+  # Construct language regex if $lang is set
+  my $lang_re;
+  if ($lang) {
+    $lang = [ $lang ] if ! ref $lang;
+    my $lang_str = join '|', sort @$lang;
+    $lang_re = qr/^($lang_str)$/;
+  }
+
+  my $count = 0;
   while (my ($long, $entry) = each %{ $self->data }) {
     # If $type is begin, restrict to 'lead' entries
     next if $type eq 'begin' && ! $entry->{lead};
     # If $lang is set, restrict to entries that include $lang
-    next if $lang && $entry->{lang} ne $lang;
+    next if $lang_re && $entry->{lang} !~ $lang_re;
 
+    $count++;
     my $long_nfd = NFD($long);
     $self->_add_to_assembler($assembler, $long_nfd);
 
@@ -156,6 +166,9 @@ sub _build_regex {
     }
   }
 
+  # If no entries found (a strange/bogus language?), return undef
+  return if $count == 0;
+
   return wantarray ? ( $assembler->re, $assembler ) : $assembler->re;
 }
 
@@ -166,8 +179,22 @@ sub regex {
   my ($type, $lang) = @_;
   $type ||= 'end';
 
+  # $lang might be an arrayref containing multiple language codes
+  my $lang_key;
+  if ($lang) {
+    $lang_key = $lang;
+    if (ref $lang && ref $lang eq 'ARRAY' && @$lang) {
+      if (@$lang == 1) {
+        $lang_key = $lang->[0];
+      }
+      else {
+        $lang_key = join '_', sort map { lc $_ } @$lang;
+      }
+    }
+  }
+
   my $cache_key = $type;
-  $cache_key .= "_$lang" if $lang;
+  $cache_key .= "_$lang_key" if $lang_key;
 
   if (my $entry = $self->regex_cache->{ $cache_key }) {
     return wantarray ? @$entry : $entry->[0];
@@ -218,22 +245,26 @@ sub split_designator {
   my ($re, $assembler) = $self->regex('end', $lang);
   my ($lead_re, $lead_assembler) = $self->regex('begin', $lang);
 
-  # Designators are usually final, so try that first
-  if ($company_name_match =~ m/^\s*(.*?)\p{XPosixPunct}*\s+($re)\s*$/) {
-    return $self->_split_designator_result($1, $2, undef, $assembler->source($^R));
-  }
-  # Not final - check for a lead designator instead (e.g. RU, NL, etc.)
-  elsif ($company_name_match =~ m/^\s*($lead_re)\p{XPosixPunct}*\s*(.*?)\s*$/) {
-    return $self->_split_designator_result(undef, $1, $2, $lead_assembler->source($^R));
-  }
-  # Not final - check for an embedded designator with trailing content
-  elsif ($company_name_match =~ m/(.*?)\p{XPosixPunct}*\s+($re)(?:\s+(.*?))?$/) {
-    return $self->_split_designator_result($1, $2, $3, $assembler->source($^R));
+  if ($re) {
+    # Handle older perls without XPosixPunct
+    my $punct_class = eval { '' =~ m/\p{XPosixPunct}/ } ? qr/\p{XPosixPunct}*/ : qr/\p{PosixPunct}*/;
+
+    # Designators are usually final, so try that first
+    if ($company_name_match =~ m/^\s*(.*?)${punct_class}\s+($re)\s*$/) {
+      return $self->_split_designator_result($1, $2, undef, $assembler->source($^R));
+    }
+    # Not final - check for a lead designator instead (e.g. RU, NL, etc.)
+    elsif ($lead_re && $company_name_match =~ m/^\s*($lead_re)${punct_class}\s*(.*?)\s*$/) {
+      return $self->_split_designator_result(undef, $1, $2, $lead_assembler->source($^R));
+    }
+    # Not final - check for an embedded designator with trailing content
+    elsif ($company_name_match =~ m/(.*?)${punct_class}\s+($re)(?:\s+(.*?))?$/) {
+      return $self->_split_designator_result($1, $2, $3, $assembler->source($^R));
+    }
   }
+
   # No match - return $company_name unchanged
-  else {
-    return $self->_split_designator_result($company_name);
-  }
+  return $self->_split_designator_result($company_name);
 }
 
 1;
@@ -372,7 +403,9 @@ designator's type.
 
 Returns a regex for all matching designators for $type ('begin'/'end') and
 $lang (iso 639-1 language code e.g. 'en', 'es', de', etc.) from the dataset.
-The regex is case-insensitive and non-anchored.
+$lang may be either a single language code scalar, or an arrayref of language
+codes, for multiple alternative languages. The returned regex is case-insensitive
+and non-anchored.
 
 $type defaults to 'end', so without parameters regex() returns a regex
 matching all designators for all languages.
@@ -390,7 +423,7 @@ standardised version of the designator as a fourth element.
 In scalar context split_designator returns a L<Business::CompanyDesignator::SplitResult>
 object.
 
-  $res = $bcd->split_designator($company_name);
+  $res = $bcd->split_designator($company_name, lang => $lang);
 
 The initial $des designator (or $res->designator)  is the designator text as
 matched in $company_name, while the final $des_std in array context (or
@@ -402,10 +435,11 @@ would find in designators() or would lookup with records(). Similarly,
 "Accessoires XYZ Ltee" (without the french acute) would match, returning
 "Ltee" (as found) for the $designator, but "Ltée" as the standardised form.
 
-split_designator also accepts an optional 'lang' parameter, which would be
-the ISO 639-1 language code for $company_name. If this is given,
-split_designator will only match designators for that language, which can
-improve the accuracy of the split.
+split_designator also accepts an optional 'lang' parameter, which defines
+one or more ISO 639-1 language codes for $company_name (can be either a single
+scalar language code, or an arrayref of alternate language codes). If $lang is
+defined, split_designator will only match designators for those languages, which
+can improve the accuracy of the split.
 
 Note that split_designator won't always get the split right. It checks for
 final designators first, then leading ones, and then finally looks for embedded
@@ -420,8 +454,8 @@ but it can also detect designators that are false positives e.g.
 
     Dr S L Ledingham - Beaumont Street Practice
 
-One way to reduce this is to specify the optional 'lang' parameter if you know
-the language your company names are in. This should reduce the number of false
+One way to mitigate this is to specify the optional 'lang' parameter if you know
+the language(s) your company names are in. This should reduce the number of false
 positives by making fewer designators available to match on, but it doesn't
 eliminate the issue altogether.
 
